@@ -414,8 +414,8 @@ class DALLE(nn.Module):
         labels = torch.cat((text, offsetted_image), dim = 1)
 
         if noncausal_attn_len > 0:
-            mask = torch.arange(seq_len, device = device)
-            mask = mask < noncausal_attn_len
+            seq_range = torch.arange(seq_len, device = device)
+            mask = seq_range < (noncausal_attn_len - 1)
             labels.masked_fill_(mask[None, :], -100) # -100 is the ignore index for cross entropy loss
 
         labels = F.pad(labels, (0, 1), value = eos_token_id) # last token predicts EOS
